import 'dart:collection';

import 'package:libaqueous/src/api/events/model/event.dart';
import 'package:libaqueous/src/api/events/model/tokens_chunk_events.dart';
import 'package:libaqueous/src/client/room/room_account_data.dart';
import 'package:libaqueous/src/client/room/room_members.dart';
import 'package:libaqueous/src/client/room/room_state.dart';
import 'package:libaqueous/src/client/user/user_state.dart';

class Store {
  Set<String> _roomIDs;
  Set<String> _userIDs;

  Map<String, RoomState> _roomStates;
  Map<String, RoomAccountData> _roomAccountData;
  Map<String, LinkedHashMap<String, Event>> _roomEvents;
  Map<String, String> _roomTokens;

  Map<String, UserState> _userStates;

  String _syncToken;

  Future<void> open() async {
    _roomIDs = Set();
    _roomStates = {};
    _roomAccountData = {};
    _roomEvents = {};

    _userIDs = Set();
    _userStates = {};
  }

  Future<void> close() async {}

  Future<void> commit() async {}

  Future<void> clear() async {}

  bool get isPermanent => false;

  Future<String> syncToken() async => _syncToken;

  Future<void> setSyncToken(String token) async {
    _syncToken = token;
  }

  Future<List<String>> roomIDs() async {
    return _roomIDs.toList();
  }

  Future<bool> containsRoomID(String id) async {
    return _roomIDs.contains(id);
  }

  Future<void> addRoomID(String id) async {
    _roomIDs.add(id);
  }

  Future<void> addRoomIDBulk(List<String> ids) async {}

  Future<RoomState> roomState(String id) async {
    return _roomStates[id];
  }

  Future<void> storeRoomState(String id, RoomState state) async {
    _roomStates[id] = state;
  }

  Future<void> storeRoomStateBulk(Map<String, RoomState> states) async {}

  Future<RoomAccountData> roomAccountData(String id) async {
    return _roomAccountData[id];
  }

  Future<void> storeRoomAccountData(String id,
      RoomAccountData accountData) async {
    _roomAccountData[id] = accountData;
  }

  Future<void> storeRoomAccountDataBulk(
      Map<String, RoomAccountData> accountDatas) async {}

  Future<RoomMembers> roomMembers(String id) async {}

  Future<void> storeRoomMembers(String id, RoomMembers roomMembers) async {}

  Future<void> storeRoomMembersBulk(Map<String, RoomMembers> bulk) async {}

  Future<void> storeLiveRoomEvent(Event event) async {
    if (event == null || event.roomID == null || event.eventID == null) return;

    if (_roomEvents[event.roomID] == null) {
      _roomEvents[event.roomID] = LinkedHashMap();
    }

    final events = _roomEvents[event.roomID];

    if (events.containsKey(event.eventID)) return;

    events[event.eventID] = event;
  }

  Future<void> storeRoomEvents(String id, TokensChunkEvents tokensChunkEvents,
      bool isHistory) async {
    if (_roomEvents[id] == null) {
      _roomEvents[id] = LinkedHashMap();
    }

    final events = _roomEvents[id];

    if (isHistory) {
      if (events.isEmpty) {
        for (final e in tokensChunkEvents.events.reversed) {
          events[e.eventID] = e;
        }

        _roomTokens[id] = tokensChunkEvents.start;
      } else {
        LinkedHashMap<String, Event> events2 = LinkedHashMap();

        for (final e in tokensChunkEvents.events.reversed) {
          events2[e.eventID] = e;
        }

        for (final e in events.values) {
          events2[e.eventID] = e;
        }
        _roomEvents[id] = events2;
      }
    }
  }

  Future<void> storeBackToken(String id, String token) {
    _roomTokens[id] = token;
  }

  Future<TokensChunkEvents> getEarlierMessages(String id, String fromToken,
      int limit) async {
    if (fromToken == _roomTokens[id] ?? null) {
      return null;
    }
    return null;
  }

  Future<void> deleteRoomData(String id) async {
    if (id == null) return;

    _roomEvents.remove(id);
    _roomTokens.remove(id);
    _roomStates.remove(id);
    _roomAccountData.remove(id);
  }

  Future<void> deleteRoom(String id) async {
    if (id == null) return;

    await deleteRoomData(id);

    _roomIDs.remove(id);
  }

  Future<List<String>> userIDs() async {
    return _userIDs.toList();
  }

  Future<bool> containsUserID(String id) async {
    return _userIDs.contains(id);
  }

  Future<void> addUserID(String id) async {
    _userIDs.add(id);
  }

  Future<void> addUserIDBulk(List<String> ids) async {}

  Future<UserState> userState(String id) async {
    return _userStates[id];
  }

  Future<void> storeUserState(String id, UserState state) async {
    _userStates[id] = state;
  }

  Future<void> storeUserStateBulk(Map<String, UserState> userStates) async {}
}
