import 'dart:io' show Platform;

import 'package:libaqueous/src/client/client.dart';
import 'package:libaqueous/src/client/store/store.dart';
import 'package:test/test.dart';

void main() {
  final String homeserver = Platform.environment['HOMESERVER'];
  final String username = Platform.environment['USERNAME'];
  final String token = Platform.environment['TOKEN'];

  group('Test Matrix Client', () {
    Client client;
    Store store;

    setUp(() async {
      store = Store();
      await store.open();
      client = Client(homeserver, username, token, store);
      await client.loadFromStore();
    });

    test('Test Client Sync', () async {
      client.startSync();
      print("Sync started.");
      await Future.delayed(const Duration(seconds: 10));
      client.stopSync();
      print("Sync stopped.");
      await store.close();
      for (final room in client.rooms.values) {
        if (room.state.isDirect) {
          print(room.id);
        }
      }
    }, timeout: Timeout(Duration(days: 1)));

    test('Test Room Post', () async {
      final room = client.rooms["!BlmKLQigsCYICQyXnd:encom.eu.org"];
      if (room != null) {
        await room.postEmote("waves from libaqueous");
      }
    });
  });
}
